/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : addr_comp.v
   Created on           : 26 Feb 99
   Function             : Address comparator for target mode
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 

   27 Feb 		  Changes to implement coding guidelines
   08 Mar         Compares only required bits in the memory address 

   Instantiated in      : devsel_gen.v

   Instantiates         :
   Synthesis check      : not done 
=========================================================================*/
`include "header.h"

module addr_comp ( 
			I_CLK,
			I_RST_N,
			I_latched_addr_lo,	// low address from AD 
			I_latched_addr_hi,	// high address from AD
			I_bar_mem_lo_addr,	
			I_bar_mem_hi_addr,
			I_bar_io_addr,

			O_lo_addr_match,
			O_hi_addr_match,
			O_io_addr_match

); 

input 		I_CLK ;
input 		I_RST_N ;

input [31:0]	I_latched_addr_lo ;
input [31:0]	I_latched_addr_hi ;
input [31:0]	I_bar_mem_lo_addr ;
input [31:0]	I_bar_mem_hi_addr ;
input [31:8]	I_bar_io_addr ;  // lower 8 bits ignored

output	O_lo_addr_match ;
output	O_hi_addr_match ;
output	O_io_addr_match ;

reg	O_lo_addr_match ;
reg	O_hi_addr_match ;
reg	O_io_addr_match ;

//  comparator for memory low address 
//  A new address is latched only at the beginning of a new command

always@(posedge I_CLK or negedge I_RST_N)
begin 

if (I_RST_N == 1'b0)
   begin 
      O_lo_addr_match <= 1'b0 ;
      O_hi_addr_match <= 1'b0 ;
      O_io_addr_match <= 1'b0 ;
   end 
else

   begin 

   if (I_latched_addr_lo[31:`mem_addr_width] ==I_bar_mem_lo_addr[31:`mem_addr_width])
      O_lo_addr_match <= 1'b1 ;
   else
      O_lo_addr_match <= 1'b0 ;

   if (I_latched_addr_hi == I_bar_mem_hi_addr)
      O_hi_addr_match <= 1'b1 ;
   else
      O_hi_addr_match <= 1'b0 ;

   if (I_latched_addr_lo[31:8] == I_bar_io_addr[31:8])
      O_io_addr_match <= 1'b1 ;
   else
      O_io_addr_match <= 1'b0 ;

   end
end

//  when the transfers are going on and the address crossed the boundary 
// supported by the device, it is an error condition. For this the address
// going out from the incrementor should be checked against base addresses.
// Yet to be implemented. Look in the spec for sequence of things to be done

endmodule

