/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : addr_gen.v 
   Created on           : 10 Mar 1999
   Function             : Generates the address for target writes/reads 
   Author               : Ramaprasad K V
   Revision             : 
   Verified             : yes/no
   Modification History : 

   Instantiated in      : tgt_addr_ctl.v

   Instantiates         :
   Synthesis check      : not done 
=========================================================================*/

// for single cycle transfers, the address bypasses the incrementor 


module addr_gen 

( 
                     I_RST_N,
                     I_CLK, // note : this comes from application side
                     I_single_cyc_xfer,
                     I_first_xfer_cyc,
                     I_dac,
                     incr_addr,
                     I_latched_addr_lo, 
                     I_latched_addr_hi, 
                     out_addr
) ;


parameter addr_width = 64 ;

input I_CLK ;
input I_RST_N ;
input I_single_cyc_xfer;
input I_first_xfer_cyc;
input I_dac;
input incr_addr ;
input [31 :0 ] I_latched_addr_hi ;
input [31 :0 ] I_latched_addr_lo ;
output [addr_width-1 :0 ] out_addr ;
   
wire [addr_width-1 : 0] out_addr ;
wire [addr_width-1 : 0] incremented_addr ;
wire [addr_width-1 : 0] load_addr ;

reg sync_first_xfer_cyc ;
reg sync_dac ;

assign out_addr = (I_single_cyc_xfer) ? {I_latched_addr_hi, I_latched_addr_lo} : incremented_addr ;
assign load_addr = {I_latched_addr_hi,I_latched_addr_lo} ;




// flops to make sure that dac and first_xfer_cyc do not miss 
// a clock edge on app_clk 

always@(negedge I_RST_N or posedge I_CLK or posedge I_dac)
begin

if (I_RST_N == 1'b0)
   sync_dac <= 1'b0 ;

else if (I_dac== 1'b1) 
   sync_dac <= 1'b1 ;

else
   sync_dac <= I_dac ;

end 

always@(negedge I_RST_N or posedge I_CLK or posedge I_first_xfer_cyc)
begin

if (I_RST_N == 1'b0)
   sync_first_xfer_cyc <= 1'b0 ;

else if (I_first_xfer_cyc== 1'b1) 
   sync_first_xfer_cyc <= 1'b1 ;

else
   sync_first_xfer_cyc <= I_first_xfer_cyc ;

end 

addr_incr addr_incr ( 
                     .I_RST_N               (I_RST_N),
                     .I_CLK                 (I_CLK),
                     .first_xfer_cycle      (sync_first_xfer_cyc),
                     .dual_addr_cycle       (sync_dac),
                     .incr_addr             (incr_addr),
                     .load_addr             (load_addr), 
                     .out_addr              (incremented_addr)
) ;


endmodule 
