/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : bar0_lo_reg.v
   Created on           : 18 Feb 1999
   Function             : Base Address register for memory mapping 
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 

   Instantiated in      : cfg_space.v

   Instantiates         :
   Synthesis check      : not done 
=========================================================================*/
/*The Memory base address lower 4 bytes is located  at address 10h 
  in the PCI configuration space. The core supports anywhere between 
  4K to 1GB (?? Not defined yet) of memory space.This size is customizable. 
  
  As per the PCI configuration mechanism, the lower 12 bits will not be
  used for the memory address decode. Bits [3:0] contain "0100" indocating
  that the memory can reside anywhere in 64-bit address space and it is 
  not prefetchable (??) . Bits [11:0] are read only and are set to 0s. Upper
  20 bits will be used for memory base address decode. Out of these bits,
  some more bits may be read only (if the device wants to address more 
  memory space). 
*/

`include "header.h"

module bar0_lo_reg ( 
                    I_CLK,
                    I_RST_N,
                    I_in_data,
                    I_reg_sel,
                    I_wr_enb,
                    I_rd_enb,
                    O_out_data,
                    O_mem_base_addr_lo
);


input   		I_CLK ; 
input   		I_RST_N ;
input [31:0] 	I_in_data ;
input   		I_reg_sel ;
input   		I_wr_enb ;
input   		I_rd_enb ;
output [31:0]  	O_out_data ;
output [31:`mem_addr_width]  	O_mem_base_addr_lo ;

wire [31:0] register_val ;
wire [31:`mem_addr_width ]  read_write_bits ;
wire [(`mem_addr_width -1) : 4] read_only_bits ;
wire [3:0] lower_four_bits ;
wire wr_enab ;

assign lower_four_bits  = 4'b0100 ;
assign read_only_bits = 0 ;
assign wr_enab = I_wr_enb & I_reg_sel ;
	
//multi_bit_reg #(32-`mem_addr_width)  U1
reg_20  U1

      (
                        .I_RST_N  (I_RST_N), 
                        .I_in_val ( I_in_data[31:`mem_addr_width] ),
                        .I_CLK (I_CLK),
                        .I_wr_enb (wr_enab),
                        .O_out_val (read_write_bits[31:`mem_addr_width] )
);


assign register_val = {read_write_bits,read_only_bits,lower_four_bits} ;
assign O_out_data  =  register_val ;
assign O_mem_base_addr_lo = register_val[31:`mem_addr_width] ;

endmodule 
