/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : bar1_reg.v
   Created on           : 18 Feb 1999
   Function             : Base Address register for I/O mapping 
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 

   Instantiated in      : cfg_space.v

   Instantiates         :
   Synthesis check      : not done 
=========================================================================*/
/* The I/O base address is located at address 18h in the PCI configuration
   space. Supports a maximum of 256 byte locations to be used as I/O. 
   So bits 31:8 should be matched in the I/O address decoder
   Bits 7:0 are read only. This will let the configuration mechanism to 
   know the size of I/O supported. Upper 24 bits are read/write. These bits
   are given to the address controller to for determining an I/O access is
   meant for the core or not.
*/

module bar1_reg ( 
                    I_CLK,
                    I_RST_N,
                    I_in_data,
                    I_reg_sel,
                    I_wr_enb,
                    I_rd_enb,
                    O_out_data,
                    O_io_base_addr
);


input   		I_CLK ; 
input   		I_RST_N ;
input [31:0] 	I_in_data ;
input   		I_reg_sel ;
input   		I_wr_enb ;
input   		I_rd_enb ;
output [31:0]  	O_out_data ;
output [31:8]  	O_io_base_addr ;

wire  [31:0] bar1_val ;
wire wr_enab ;

assign bar1_val[1:0] = 2'b01 ; // Indicates it is a IO register
assign bar1_val[7:2] = 6'b000000 ; // For 256 bytes of IO space

assign wr_enab = I_wr_enb & I_reg_sel ;

//multi_bit_reg #(24)  U1
reg_24 U1

      (
                        .I_RST_N (I_RST_N), 
                        .I_in_val ( I_in_data[31:8]),
                        .I_CLK (I_CLK),
                        .I_wr_enb (wr_enab),
                        .O_out_val (bar1_val[31:8])
);

assign O_io_base_addr = bar1_val[31:8] ;
assign O_out_data = bar1_val ;

endmodule 
