/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : cmd_sts_reg.v
   Created on           : 17 Feb 1999
   Function             : PCI Command & ststus register at address 04h
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 

   Instantiated in      : cfg_space.v

   Instantiates         :cmd_reg.v, sts_reg.v
   Synthesis check      : not done 
=========================================================================*/

module cmd_sts_reg(
                     I_CLK,
                     I_RST,
                     I_in_data,
                     I_cs_reg_sel,
					 I_wr_enb,
					 I_rd_enb,
                     set_data_prty_err_reported, // bit 8
                     set_signalled_system_error, // bit 11
                     set_received_master_abort,  // bit 12
                     set_received_target_abort,  // bit 13
                     set_signalled_target_abort, // bit 14
                     set_data_prty_err_detected, // bit 15

                     O_out_data,
                     O_io_spc_enb,     
                     O_mem_spc_enb,  
                     O_prty_err_resp, 
                     O_master_enb,    
                     O_serr_enb     
);


input     I_CLK;
input     I_RST;
input     I_cs_reg_sel;
input     I_wr_enb;
input	  I_rd_enb;
input     set_data_prty_err_reported; // bit 8
input     set_signalled_system_error; // bit 11
input     set_received_master_abort;  // bit 12
input     set_received_target_abort;  // bit 13
input     set_signalled_target_abort; // bit 14
input     set_data_prty_err_detected; // bit 15

input  [31:0]   I_in_data;
output [31:0]   O_out_data ;
output			O_io_spc_enb;     
output 			O_mem_spc_enb;  
output 			O_prty_err_resp; 
output 			O_master_enb;    
output			O_serr_enb;    

wire [15:0] I_in_data_lo ;
wire [15:0] O_out_data_lo ;
wire [31:16] I_in_data_hi ;
wire [31:16] O_out_data_hi ;

assign I_in_data_lo = I_in_data[15:0];
assign I_in_data_hi = I_in_data[31:16] ;

assign O_out_data = {O_out_data_hi, O_out_data_lo};

cmd_reg U1 (
                    .I_CLK (I_CLK),
                    .I_RST (I_RST),
                    .I_in_data (I_in_data_lo),
                    .reg_sel (I_cs_reg_sel),
                    .I_wr_enb (I_wr_enb),
                    .I_rd_enb (I_rd_enb),
                    .O_out_data (O_out_data_lo),
                    .O_io_spc_enb (O_io_spc_enb),     
                    .O_mem_spc_enb (O_mem_spc_enb),  
                    .O_prty_err_resp ( O_prty_err_resp), 
                    .O_master_enb (O_master_enb),    
                    .O_serr_enb (O_serr_enb)     
 ); 

status_reg U2 (
                    I_CLK,
                    I_RST,
                    I_cs_reg_sel,
                    I_wr_enb,
                    I_rd_enb,
                    I_in_data_hi,
                    set_data_prty_err_reported, // bit 8
                    set_signalled_system_error, // bit 11
                    set_received_master_abort,  // bit 12
                    set_received_target_abort,  // bit 13
                    set_signalled_target_abort, // bit 14
                    set_data_prty_err_detected, // bit 15
                    
                    O_out_data_hi
 );

endmodule
