/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : lat_timer_reg.v
   Created on           : 17 Feb 1999
   Function             : Latency timer register for PCI core
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 
   
   Instantiated in      : cfg_space.v

   Instantiates         : reg_bit_0.v, reg_bit_1.v
   Synthesis check      : not done 
=========================================================================*/

// The latency timer register is located at 0Ch in the configuration space
// Of the 4 bytes in this register, 3 (MS byte),2 & 0 are read only and 
// return a value 0.( un-implemented bits)
// Byte 1 contains the latency timer value. Default value is 8 cycles
// Lower 2 bits of this byte are always 0 
// bit 23 : header type = 0 , indicating a single function PCI device

module lat_timer_reg (

                       I_RST_N,
                       I_CLK,
                       I_reg_sel,
                       I_in_data,
                       I_wr_enb,
                       O_timer_val,
                       O_out_data

);

input			I_RST_N;
input			I_CLK;
input			I_reg_sel;
input			I_wr_enb;
input  [31:0]	I_in_data;
output [31:0]	O_out_data;
output [7:0]    O_timer_val ; // This indication goes to initiator engine

wire [31:0] timer_reg_val ;
wire wr_enb ;
wire rd_enb ;

assign O_out_data = timer_reg_val ;
assign wr_enb = I_wr_enb & I_reg_sel ;

assign timer_reg_val[31:16] = 16'h0000 ;
assign timer_reg_val[10:0] = 11'b00000000000 ;
assign O_timer_val = timer_reg_val[15:8] ;

reg_bit_0 bit15 (
                I_RST_N,
                I_in_data[15],
                I_CLK,
                wr_enb,
                timer_reg_val[15]);

reg_bit_0 bit14 (
                I_RST_N,
                I_in_data[14],
                I_CLK,
                wr_enb,
                timer_reg_val[14]);


reg_bit_0 bit13 (
                I_RST_N,
                I_in_data[13],
                I_CLK,
                wr_enb,
                timer_reg_val[13]);


reg_bit_0 bit12 (
                I_RST_N,
                I_in_data[12],
                I_CLK,
                wr_enb,
                timer_reg_val[12]);

reg_bit_1 bit11 (
                I_RST_N,
                I_in_data[11],
                I_CLK,
                wr_enb,
                timer_reg_val[11]);
endmodule
