/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : status_reg.v
   Created on           : 17 Feb 1999
   Function             : PCI status register at address 04h;higher 2 bytes
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 

   4th Mar 99	          No tristates 
   Instantiated in      : cfg_space.v

   Instantiates         : sts_bit_0.v 
   Synthesis check      : not done 
===========================================================================*/

// This is the status register at address 04h ; 
// Contains the higher two bytes at that address ;
// The lower two bytes are occupied by command register

module status_reg (
                    I_CLK,
                    I_RST,
                    I_reg_sel,
                    wr_enb,
                    rd_enb,
                    I_in_data,
                    set_data_prty_err_reported, // bit 8
                    set_signalled_system_error, // bit 11
                    set_received_master_abort,  // bit 12
                    set_received_target_abort,  // bit 13
                    set_signalled_target_abort, // bit 14
                    set_data_prty_err_detected, // bit 15
                    
                    O_out_data
 );

input 		I_CLK ;
input 		I_RST ;
input 		I_reg_sel ;
input 		rd_enb ;
input 		wr_enb ;
input 		set_data_prty_err_reported ;
input		set_signalled_system_error ;
input		set_received_master_abort ;
input		set_received_target_abort ;
input		set_signalled_target_abort ;
input		set_data_prty_err_detected  ;

input  [31:16]  I_in_data ;
output [31:16] 	O_out_data ;

wire [7:0] low_byte ;
wire [1:0] devsel_timing ; 
wire [31:16] sts_reg_val ;

assign sts_reg_val[23:16] = 8'b00100000 ;   

// 23 : Fast back to back enable as master not available 0
// 22 : Link list for user defined features not available 0
// 21 : 66 MHz capability 1  
// 21:16 : Reserved bits 00000

assign sts_reg_val[26:25] = 2'b10 ;
//	devsel_timing = 10 indicating slow decode speed


//assign O_out_data [31:16] = (rd_enb & I_reg_sel) ? sts_reg_val : 32'bz ;
assign O_out_data [31:16] = sts_reg_val  ;


sts_bit_0 bit8(I_RST,
             set_data_prty_err_reported,
             I_in_data[24],
             I_CLK,
             wr_enb,
             rd_enb,
             sts_reg_val[24] ); //8 + 16

sts_bit_0 bit11(I_RST,
             set_signalled_system_error,
             I_in_data[27],
             I_CLK,
             wr_enb,
             rd_enb,
             sts_reg_val[27] ); // 11 + 16

               
sts_bit_0 bit12(I_RST,
             set_received_master_abort,
             I_in_data[28],
             I_CLK,
             wr_enb,
             rd_enb,
             sts_reg_val[28] );

sts_bit_0 bit13(I_RST,
             set_received_target_abort,
             I_in_data[29],
             I_CLK,
             wr_enb,
             rd_enb,
             sts_reg_val[29] );


sts_bit_0 bit14(I_RST,
             set_signalled_target_abort,
             I_in_data[30],
             I_CLK,
             wr_enb,
             rd_enb,
             sts_reg_val[30] );


sts_bit_0 bit15(I_RST,
             set_data_prty_err_detected,
             I_in_data[31],
             I_CLK,
             wr_enb,
             rd_enb,
             sts_reg_val[31] );

endmodule
