/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : tgt_misc.v
   Created on           : 26 Feb 99
   Function             : Miscellaneous logic 
   Author               : Ramaprasad K V
   Revision             : 
   Verified             : no
   Modification History : 

   Instantiated in      : 

   Instantiates         :
   Synthesis check      : not done 
=========================================================================*/

// This block contains miscellaneous logic that could not be included
// in other blocks. If this becomes a huge block, see that it is properly
// divided into different modules



module tgt_misc ( 
					I_RST_N, 
					I_CLK,
					I_FRAME_N,		// frame and trdy should be directly from the PCI bus
					I_IRDY_N,
					I_trdy_N,
                    O_latched_irdy,
					O_last_xfer_flag 
);

input I_RST_N ;
input I_CLK ;
input I_FRAME_N ;
input I_IRDY_N ;
input I_trdy_N ;

output O_last_xfer_flag ; 
output O_latched_irdy ; 
wire O_last_xfer_flag ; 
reg O_latched_irdy ; 
reg fifo_wr_enb_1 ;
reg fifo_wr_enb_2 ;

reg prev_frame ;
wire last_xfer ; 
reg extended_last_xfer ;

// a combinatorial flag 
//assign O_last_xfer_flag =  I_FRAME_N & (!I_trdy_N) & (!I_IRDY_N) ;
assign O_last_xfer_flag = last_xfer | extended_last_xfer ; // will remain high till a xfer is done 

always@(posedge I_CLK  or negedge I_RST_N)
begin

if (I_RST_N == 1'b0 )
   prev_frame <= 1'b1 ;
else
   prev_frame <= I_FRAME_N ;
end 

assign last_xfer = I_FRAME_N & ~prev_frame ; 

// In case when frame is deasserted, if either trdy or irdy are deasserted,
// the last transfer indication has to be extended  so that the target sm 
// knows when to stop driving the bus 

always@(posedge I_CLK  or negedge I_RST_N)
begin

if (I_RST_N == 1'b0 )
   extended_last_xfer <= 1'b0 ;
else
   begin 
   if (last_xfer) // when time for last transfer 
      if (I_IRDY_N  || I_trdy_N ) // one of them de-asserted 
         extended_last_xfer <= 1'b1 ;
      else
         extended_last_xfer <= 1'b0 ;

   if (extended_last_xfer) // already high 
      if (!I_IRDY_N && !I_trdy_N) // both assrerted for a cycle  -- may not happen 
                                    // when the last_xfer comes when data is in the pipeline
                                    // already and no more reads need to be done 
         extended_last_xfer <= 1'b0 ; // reset condition 
   else if (!I_FRAME_N && prev_frame) // beginning of a new frame 
         extended_last_xfer <= 1'b0 ; 
   end
end


always@(posedge I_CLK  or negedge I_RST_N)
begin

if (I_RST_N == 1'b0 )

   O_latched_irdy <= 1'b1 ;
else 
   O_latched_irdy <= I_IRDY_N ;
end 


always@(posedge I_CLK  or negedge I_RST_N)
begin

if (I_RST_N == 1'b0 )
   fifo_wr_enb_1 <= 1'b0 ;
else
   if (!I_IRDY_N  && !I_trdy_N)
      fifo_wr_enb_1 <= 1'b1 ;
   else
      fifo_wr_enb_1 <= 1'b0 ;
end


always@(posedge I_CLK  or negedge I_RST_N)
begin

if (I_RST_N == 1'b0 )
   fifo_wr_enb_2 <= 1'b0 ;
else
   if (!I_IRDY_N  && !I_trdy_N)
      fifo_wr_enb_2 <= 1'b1 ;
   else
      fifo_wr_enb_2 <= 1'b0 ;
end

endmodule 
