/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : tgt_top.v
   Created on           : 3 Mar 99
   Function             : Top level for 
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 
   8th March              instanced tgt_wr_dp block
   10th Mar               instantiating tgt_fifo module .. expt
   15 Mar 99              do_disc implemented for improper burst orders   
   18 Mar 99              moving top level glue logic into a module
   19 Mar 99              parity check functionality introduced (par_check)

   Instantiated in      : CLPCI core

   Instantiates         : tgt_cmd_decode.v, tgt_addr_ctl.v , cfg_space.v,
                          tgt_engine.v
   Synthesis check      : not done 
=========================================================================*/

// This module envolopes all target functions. Has no inout ports. That 
// has to be maintained at clpci core level ( managing the in outs )

`include "header.h"

module tgt_top (

						I_RST_N,
						I_CLK,
						I_FRAME_N,
						I_REQ64_N,
						I_CBE_N,
						I_IDSEL,
						I_IRDY_N, 

                        I_app_clk,     						
                        I_tgt_wr_nxt,
						I_capable64,
						I_disc_from_app,
						I_abort_from_app,
						I_back_end_busy, // when xfers on back end bus 
			
                        I_data_hi,
                        I_data_lo,
                        I_data_valid,

						O_STOP_N,
						O_TRDY_N,
						O_DEVSEL_N,
						O_ACK64_N,
					
						IO_AD ,
                        IO_PAR,
                        IO_PAR64,

                        O_tgt_wr_data_avail,
                        O_tgt_do_rd_xfer,  
                        O_data_lo,        
                        O_data_hi,       
                        O_addr_lo,
                        O_addr_hi,       
                        O_data_valid_lo,   // validates both address and data       
                        O_data_valid_hi      
);


input			I_RST_N;
input			I_CLK;
input			I_FRAME_N;
input			I_REQ64_N;
input	[3:0] 	I_CBE_N;
input			I_IDSEL;
input			I_IRDY_N;


inout	[63:0]	IO_AD;  
inout           IO_PAR ;
inout           IO_PAR64 ;


input           I_app_clk;
input           I_tgt_wr_nxt;
input			I_capable64 ;
input			I_disc_from_app ;
input			I_abort_from_app ;
input			I_back_end_busy ;

input   [31:0]  I_data_hi ;
input   [31:0]  I_data_lo ;
input           I_data_valid ;


output			O_STOP_N ;
output			O_TRDY_N ;
output			O_DEVSEL_N ;
output			O_ACK64_N ;

output          O_tgt_wr_data_avail ;
output          O_tgt_do_rd_xfer ;
output  [31:0]  O_data_hi ;
output  [31:0]  O_data_lo ;
output  [31:0]  O_addr_hi ;
output  [31:0]  O_addr_lo ;
output          O_data_valid_lo ;
output          O_data_valid_hi ;



wire 	io_rd ;
wire 	io_wr ;
wire 	mem_rd ;
wire 	mem_wr ;
wire	cfg_rd ;
wire	cfg_wr ;
wire	dac ;

wire  	[31:0] 	in_data_bus_lo ; // internal uni-directional data busses
wire  	[31:0] 	in_data_bus_hi ;
wire  	[31:0] 	out_data_bus_lo ;
wire  	[31:0] 	rd_fifo_data_lo ;
wire  	[31:0] 	rd_fifo_data_hi ;
wire  	[31:0] 	data_out_from_cfg ;

wire	[31:0]	bar_mem_lo_addr ;	
wire	[31:0]	bar_mem_hi_addr ;	
wire 	[63:`mem_addr_width]     mem_base_addr;
wire 	[31:8] 	io_base_addr ;
		     
wire			valid_cfg_rd ;     
wire	 		valid_cfg_wr ;     

wire	        device_reg_sel ;
wire	        cs_reg_sel ;
wire	        cc_reg_sel ;
wire	        lt_reg_sel ;
wire        	bar0_lo_reg_sel ;
wire        	bar0_hi_reg_sel ;
wire	   		bar1_reg_sel  ;
wire	   		intr_reg_sel  ;

wire			first_xfer_cyc  ;
wire			second_xfer_cyc ;
wire			single_cyc_xfer ;

wire            set_prty_err_reported;
wire            set_detected_prty_err;    
wire            set_signalled_tgt_abort ;
wire            set_recd_tgt_abort;     
wire            set_recd_mstr_abort; 
wire            set_signalled_serr;   

wire            io_spc_enb ;    
wire            mem_spc_enb;  
                    
wire            prty_err_resp; 
wire            master_enb    ;
wire            serr_enb     ;
wire [7:0]      lat_timer_val;
wire			latched_frame ;
wire			latched_req64 ;
wire 			hi_addr_match ;                  
wire 			lo_addr_match ;                  
wire 			io_addr_match ;                  

wire            calcd_par ;
wire            calcd_par64 ;

wire            par_from_pci ;
wire            par64_from_pci ;

wire			twf_full ; // from the FIFO control block which will be 
wire			trf_empty ;  // instantiated here later
wire			trf_not_full ; // for generating O_tgt_do_rd_xfer

wire 			incr_twf_wrptr ;
wire 			incr_trf_rdptr ;

wire			stop_N ;
wire			trdy_N ;
wire			devsel_N ;
wire 			ack64_N ;
wire			oe_stop ;
wire			oe_trdy ;
wire			oe_devsel ;
wire 			oe_ack64 ;
wire			oe_PAR ;
wire			oe_PAR64;
wire 			oe_AD_lo ;
wire 			oe_AD_hi ;

wire [`mem_addr_width -1 :0 ] trailing_zeros ;
wire [31:0]     twf_data_lo ; // write data going to target write FIFO
wire [63:32]    twf_data_hi ;

wire [31:0]     trf_data_lo ; // read data from the target read FIFO
wire [63:32]    trf_data_hi ;

wire           incr_addr_to_app ; // should be a function of read and write on app side
wire            read_mode ; // 1 while reads going on; Useful for generating O_tgt_do_rd_xfer
wire            end_xfer ; 
wire            fifo_reset ; // use to reset fifo pointers  in read mode 

wire do_disc ;
wire reg_tgt_wr_nxt ; // WIll be used fro reading from TWF fifo
wire r_incr_twf_wrptr ;
wire addr_par_err ; // this should have an output enable = first_xfer_cycle 
wire [3:0] latched_CBE ;

tgt_cmd_decode  tgt_cmd_decode (
						.I_RST_N 	(I_RST_N) ,
						.I_CLK 		(I_CLK) ,
						.I_FRAME_N 	(I_FRAME_N) ,
						.I_CBE_N 	(I_CBE_N ),
						.I_IDSEL 	(I_IDSEL) ,
                        
                        .O_io_rd 	(io_rd ),
                        .O_io_wr 	(io_wr ),
                        .O_mem_rd 	(mem_rd ),
                        .O_mem_wr 	(mem_wr) ,
                        .O_cfg_rd 	(cfg_rd ),
                        .O_cfg_wr 	(cfg_wr) ,
                        .O_dac 		(dac)

 );

tgt_addr_ctl  tgt_addr_ctl (
						.I_RST_N 	(I_RST_N) ,
						.I_CLK 		(I_CLK) ,
						.I_FRAME_N 	(I_FRAME_N) ,
						.I_REQ64_N 	(I_REQ64_N) ,

                        .I_AD 		(in_data_bus_lo) ,
						.I_CBE      (I_CBE_N),
						.I_IRDY_N 	(I_IRDY_N) , 
						.I_trdy_N 	(trdy_N) ,				// this is an internal signal 
						.I_par_from_pci (par_from_pci),

						.I_cfg_rd 	(cfg_rd) ,				// from command decoder
						.I_cfg_wr 	(cfg_wr) ,
						.I_dac 		(dac) ,

						.I_bar_mem_lo_addr 	(bar_mem_lo_addr) ,	// From config space
						.I_bar_mem_hi_addr 	(bar_mem_hi_addr) ,	
						.I_bar_io_addr 		(io_base_addr) ,
                        .I_incr_addr        (incr_addr_to_app),
  		                .I_app_clk          (I_app_clk),
     
						.O_valid_cfg_rd 	(valid_cfg_rd) ,     
						.O_valid_cfg_wr 	(valid_cfg_wr) ,     

   		                .O_device_reg_sel 	(device_reg_sel) ,
   		                .O_cs_reg_sel 		(cs_reg_sel) ,
   		                .O_cc_reg_sel 		(cc_reg_sel) ,
   		                .O_lt_reg_sel 		(lt_reg_sel) ,
   		                .O_bar0_lo_reg_sel 	(bar0_lo_reg_sel) ,
   		                .O_bar0_hi_reg_sel 	(bar0_hi_reg_sel) ,
	                 	.O_bar1_reg_sel 	(bar1_reg_sel) ,
	                 	.O_intr_reg_sel 	(intr_reg_sel) ,

						.O_latched_frame	(latched_frame),
					    .O_req_64bit_xfer	(latched_req64),
						.O_first_xfer_cyc 	(first_xfer_cyc) , 
						.O_second_xfer_cyc 	(second_xfer_cyc) , 
						.O_single_cyc_xfer 	(single_cyc_xfer) , 

						.O_addr_lo 	        (O_addr_lo), 
						.O_addr_hi 	        (O_addr_hi) ,
                		.O_lo_addr_match	(lo_addr_match),
               			.O_hi_addr_match	(hi_addr_match),
                		.O_io_addr_match	(io_addr_match),
                        .O_addr_par_err     (addr_par_err),
                        .O_do_disc          (do_disc)
					
);


cfg_space  cfg_space (
                    .I_in_data_bus 			(in_data_bus_lo),
                    .I_CLK 					(I_CLK ),
                    .I_RST_N 				(I_RST_N ),

                    .I_wr_enb 				(valid_cfg_wr ),
                    .I_rd_enb 				(valid_cfg_rd ),
					.I_trdy_N			    (trdy_N),

                    .I_device_reg_sel 		(device_reg_sel ),  		
                    .I_cs_reg_sel 			(cs_reg_sel ),			
                    .I_cc_reg_sel 			(cc_reg_sel ),			
                    .I_lt_reg_sel 			(lt_reg_sel ),		
                    .I_bar0_lo_reg_sel 		(bar0_lo_reg_sel ),
                    .I_bar0_hi_reg_sel 		(bar0_hi_reg_sel ),
                    .I_bar1_reg_sel 		(bar1_reg_sel ),
                    .I_intr_reg_sel 		(intr_reg_sel ),

                    .I_set_prty_err_reported 	(set_prty_err_reported ),
                    .I_set_detected_prty_err 	(set_detected_prty_err ),    

                    .I_set_signalled_tgt_abort 	(set_signalled_tgt_abort  ),

                    .I_set_recd_tgt_abort 		(set_recd_tgt_abort ),     
                    .I_set_recd_mstr_abort 		(set_recd_mstr_abort ), 
                    .I_set_signalled_serr 		(set_signalled_serr ),   

                    .O_io_spc_enb 			(io_spc_enb ),     
                    .O_mem_spc_enb 			(mem_spc_enb ),  
                    
                    .O_prty_err_resp 		(prty_err_resp ), 
                    .O_master_enb 			(master_enb ),    
                    .O_serr_enb 			(serr_enb ),     
                    .O_lat_timer_val 		(lat_timer_val ),

                    .O_mem_base_addr 		(mem_base_addr ),
                    .O_io_base_addr 		(io_base_addr ), 
                    
                    .O_out_data_bus 		(data_out_from_cfg)

 );


tgt_wr_dp tgt_wr_dp (

					.I_RST_N            (I_RST_N),
					.I_CLK              (I_CLK),

                    .I_in_data_lo       (in_data_bus_lo),
                    .I_in_data_hi       (in_data_bus_hi),
                    .I_devsel_N         (devsel_N) ,

                    .O_latched_data_lo  (twf_data_lo),  // will go to the data ports of twf fifo
                    .O_latched_data_hi  (twf_data_hi)
);

tgt_rd_dp tgt_rd_dp (

					.I_RST_N            (I_RST_N),
					.I_CLK              (I_CLK),

                    .I_in_data_lo       (trf_data_lo),  // from the data port of TRF fifo
                    .I_in_data_hi       (trf_data_hi),
                    .I_devsel_N         (devsel_N) ,

                    .O_latched_data_lo  (rd_fifo_data_lo),  // indicating this is data from read FIFO
                    .O_latched_data_hi  (rd_fifo_data_hi),
					.O_par              (calcd_par),
					.O_par64            (calcd_par64)
);


tgt_engine  tgt_engine (


				.I_RST_N				(I_RST_N),
				.I_CLK					(I_CLK),
                .I_FRAME_N				(I_FRAME_N),	
                .I_frame_N				(latched_frame),
 				.I_REQ64_N 				(I_REQ64_N),
				.I_IRDY_N				(I_IRDY_N),		    // note 
		 		.I_capable64 			(I_capable64),

				.I_disc_from_app		(I_disc_from_app),
				.I_abort_from_app		(I_abort_from_app),

			
				.I_first_xfer_cyc		(first_xfer_cyc),		
				.I_second_xfer_cyc		(second_xfer_cyc),		
				.I_single_cyc_xfer		(single_cyc_xfer),		
                .I_lo_addr_match		(lo_addr_match),
               	.I_hi_addr_match		(hi_addr_match),
                .I_io_addr_match		(io_addr_match),
                .I_do_disc              (do_disc),

				.I_io_rd				(io_rd), // From Target command decoder
                .I_io_wr				(io_wr),				
                .I_mem_rd				(mem_rd),			
                .I_mem_wr				(mem_wr),		
                .I_dac					(dac),         
				.I_valid_cfg_rd 		(valid_cfg_rd),
				.I_valid_cfg_wr 		(valid_cfg_wr),
	
				.I_io_spc_enb			(io_spc_enb),
				.I_mem_spc_enb			(mem_spc_enb),
			
				.I_back_end_busy		(I_back_end_busy),

				.I_twf_full				(twf_full),
				.I_trf_empty			(trf_empty),

                .O_read_mode            (read_mode),
                .O_end_xfer             (end_xfer),
				.O_stop_N 				(stop_N),
				.O_oe_stop 				(oe_stop),
				.O_devsel_N 			(devsel_N),
				.O_oe_devsel 			(oe_devsel),
				.O_trdy_N 				(trdy_N),
				.O_oe_trdy 				(oe_trdy),
				.O_ack64_N 				(ack64_N),
				.O_oe_ack64 			(oe_ack64),
				.O_oe_AD_lo				(oe_AD_lo),
				.O_oe_AD_hi				(oe_AD_hi),
                .O_oe_PAR 				(oe_PAR),
                .O_oe_PAR64				(oe_PAR64), 	
				.O_incr_twf_wrptr		(incr_twf_wrptr),
				.O_incr_trf_rdptr		(incr_trf_rdptr)


) ;



// parity check module 


par_check par_check (


   .I_RST_N                 (I_RST_N),
   .I_CLK                   (I_CLK),
   .I_incr_twf_wrptr        (r_incr_twf_wrptr) , 
   .I_par_from_pci          (par_from_pci),
   .I_par64_from_pci        (par64_from_pci),
   .I_ack64_N               (ack64_N),
   .I_twf_data_hi           (twf_data_hi),
   .I_twf_data_lo           (twf_data_lo),
   .I_prty_err_resp         (prty_err_resp),

   .O_set_detected_prty_err (set_detected_prty_err),     // should be set even if prty_err_resp bit is 0
   .O_perr                  (O_perr)        // wire now, later should reach the ports

);



tgt_fifo tgt_fifo
(
// inputs from PCI side 

   .I_RST_N                  (I_RST_N),
   .I_CLK                    (I_CLK),
   .I_fifo_reset             (fifo_reset),
//   .I_fifo_reset             (1'b0),
   .I_twf_data_lo            (twf_data_lo),
   .I_twf_data_hi            (twf_data_hi),
   .I_incr_twf_wrptr         (r_incr_twf_wrptr),  // to be in synch with the data 
   .I_incr_trf_rdptr         (incr_trf_rdptr),

// inputs from app side 

   .I_app_clk                (I_app_clk),
   .I_tgt_wr_nxt             (reg_tgt_wr_nxt),
   .I_data_lo                (I_data_lo),
   .I_data_hi                (I_data_hi),
   .I_data_valid             (I_data_valid),

// outputs to PCI side 
   .O_trf_data_lo            (trf_data_lo),
   .O_trf_data_hi            (trf_data_hi),
   .O_trf_empty              (trf_empty),
   .O_twf_full               (twf_full),

// outputs to app side 

   .O_tgt_wr_data_avail      (O_tgt_wr_data_avail),  
   .O_trf_not_full           (trf_not_full),
   .O_data_lo                (O_data_lo),
   .O_data_hi                (O_data_hi)
    
);


top_glue glue  (

                   // signlas from the pci bus

   .I_RST_N              (I_RST_N),
   .I_CLK                (I_CLK),
   .I_FRAME_N            (I_FRAME_N),
   .I_REQ64_N            (I_REQ64_N),
   .I_IRDY_N             (I_IRDY_N),

          // signals from the core internal 

   .I_stop_N             (stop_N),      // from target engine 
   .I_trdy_N             (trdy_N), 
   .I_devsel_N           (devsel_N), 
   .I_ack64_N            (ack64_N),
   
   .I_oe_stop            (oe_stop),
   .I_oe_trdy            (oe_trdy),
   .I_oe_devsel          (oe_devsel),
   .I_oe_ack64           (oe_ack64),
  
   .I_oe_AD_hi           (oe_AD_hi),
   .I_oe_AD_lo           (oe_AD_lo),

   .I_tgt_wr_nxt         (I_tgt_wr_nxt),
   .I_calcd_par          (calcd_par),
   .I_calcd_par64        (calcd_par64),
			
   .I_rd_fifo_data_lo    (rd_fifo_data_lo),       // fifo data from the dp block (registered0
   .I_rd_fifo_data_hi    (rd_fifo_data_hi),

   .I_data_out_from_cfg  (data_out_from_cfg),
   .I_valid_cfg_rd       (valid_cfg_rd),

   .I_trf_not_full        (trf_not_full),
   .I_incr_twf_wrptr      (incr_twf_wrptr),

   .I_read_mode           (read_mode),
   .I_end_xfer            (end_xfer),
   .I_single_cyc_xfer     (single_cyc_xfer),

                 // signals form the application i/f

   .I_app_clk             (I_app_clk),
   .I_data_valid          (I_data_valid),
   .I_tgt_wr_data_avail   (O_tgt_wr_data_avail),

         // outputs into target internal modules 
 
   .O_in_data_bus_lo      (in_data_bus_lo),
   .O_in_data_bus_hi      (in_data_bus_hi),
   .O_in_par              (par_from_pci),
   .O_in_par64            (par64_from_pci),
   .O_fifo_reset          (fifo_reset),
   .O_incr_addr_to_app    (incr_addr_to_app),
   .O_reg_tgt_wr_nxt      (reg_tgt_wr_nxt),
   .O_r_incr_twf_wrptr    (r_incr_twf_wrptr),
      
           // outputs to app i/f 

   .O_tgt_do_rd_xfer      (O_tgt_do_rd_xfer),  
   .O_data_valid_lo       (O_data_valid_lo),   // validates both address and data       
   .O_data_valid_hi       (O_data_valid_hi),

                // outputs to PCI i/f
   .O_STOP_N              (O_STOP_N),
   .O_TRDY_N              (O_TRDY_N),
   .O_DEVSEL_N            (O_DEVSEL_N),
   .O_ACK64_N             (O_ACK64_N),
			
   .IO_AD                 (IO_AD),
   .IO_PAR                (IO_PAR),
   .IO_PAR64              (IO_PAR64)
   
);

// This section is deliberately kept here instead of top_glue. 

assign trailing_zeros  = 0 ;
assign bar_mem_hi_addr = mem_base_addr[63:32] ;
assign bar_mem_lo_addr = { mem_base_addr[31:`mem_addr_width], trailing_zeros} ; 
// append trailing, not leading 0's !!

endmodule 
