/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : top_glue.v
   Created on           : 17 Mar 99
   Function             : Glue logic in target top level  
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 
   28 March               Added cbe to ports 

   Instantiated in      : tgt_top.v 

   Instantiates         : 
   Synthesis check      : not done 
=========================================================================*/


`include "header.h"

module top_glue  (

                   // signlas from the pci bus

   I_RST_N,
   I_CLK,
   I_FRAME_N,
   I_REQ64_N,
   I_IRDY_N,

          // signals from the core internal 

   I_stop_N,      // from target engine 
   I_trdy_N, 
   I_devsel_N, 
   I_ack64_N,
   
   I_oe_stop,
   I_oe_trdy,
   I_oe_devsel,
   I_oe_ack64,
  
   I_oe_AD_hi,
   I_oe_AD_lo,

   I_tgt_wr_nxt,
   I_calcd_par,
   I_calcd_par64,
			
   I_rd_fifo_data_lo,  // fifo data from the dp block (registered0
   I_rd_fifo_data_hi,

   I_data_out_from_cfg,
   I_valid_cfg_rd,

   I_trf_not_full,
   I_incr_twf_wrptr,

   I_read_mode,
   I_end_xfer,
   I_single_cyc_xfer,

                 // signals form the application i/f

   I_app_clk,
   I_data_valid,
   I_tgt_wr_data_avail,

         // outputs into target internal modules 
 
   O_in_data_bus_lo,
   O_in_data_bus_hi,
   O_in_par,
   O_in_par64,
   O_fifo_reset,
   O_incr_addr_to_app,
   O_reg_tgt_wr_nxt,
   O_r_incr_twf_wrptr,

                 // outputs to app i/f 

   O_tgt_do_rd_xfer,  
   O_data_valid_lo,   // validates both address and data       
   O_data_valid_hi ,     

                // outputs to PCI i/f
   O_STOP_N,
   O_TRDY_N,
   O_DEVSEL_N,
   O_ACK64_N,
			    // inouts to PCI i/f
   IO_AD ,
   IO_PAR,
   IO_PAR64,
   IO_CBE_lo,
   IO_CBE_hi
   
);


input	I_RST_N;
input	I_CLK;
input	I_FRAME_N;
input	I_REQ64_N;
input	I_IRDY_N;

input   I_calcd_par;
input   I_calcd_par64;

input   I_stop_N; 
input   I_trdy_N; 
input   I_devsel_N; 
input   I_ack64_N;
   
input   I_oe_stop;
input   I_oe_trdy;
input   I_oe_devsel;
input   I_oe_ack64;
input   I_oe_AD_hi;
input   I_oe_AD_lo;


inout	[63:0]	IO_AD;  
inout           IO_PAR;
inout           IO_PAR64;
inout   [3:0]   IO_CBE_lo;
inout   [3:0]   IO_CBE_hi;

input           I_app_clk;
input           I_tgt_wr_nxt;

input           I_data_valid ;


output			O_STOP_N ;
output			O_TRDY_N ;
output			O_DEVSEL_N ;
output			O_ACK64_N ;

input           I_tgt_wr_data_avail ;
input  	[31:0] 	I_rd_fifo_data_lo ;
input  	[31:0] 	I_rd_fifo_data_hi ;
input  	[31:0] 	I_data_out_from_cfg ;
input           I_valid_cfg_rd ;
input			I_trf_not_full ; // for generating O_tgt_do_rd_xfer
input 			I_incr_twf_wrptr ;
input           I_read_mode ;
input           I_end_xfer ;
input           I_single_cyc_xfer ;


output          O_tgt_do_rd_xfer ;
output          O_data_valid_lo ;
output          O_data_valid_hi ;
output          O_in_par ;
output          O_in_par64 ;

output          O_fifo_reset ; // use to reset fifo pointers  in read mode 
output          O_incr_addr_to_app ;
output          O_reg_tgt_wr_nxt ;
output          O_r_incr_twf_wrptr ;

reg     O_data_valid_hi ; 
reg     O_data_valid_lo ; 
reg     O_fifo_reset ; // use to reset fifo pointers  in read mode 
reg     O_reg_tgt_wr_nxt ; // WIll be used fro reading from TWF fifo
reg     O_r_incr_twf_wrptr ;


output 	[31:0] 	O_in_data_bus_lo ; // internal uni-directional data busses
output 	[31:0] 	O_in_data_bus_hi ;

wire  	[31:0] 	out_data_bus_lo ; // for muxing between fifo data or config data

wire	[31:0]	bar_mem_lo_addr ;	
wire	[31:0]	bar_mem_hi_addr ;	
		     
// internal regs
//  synchronizing trdy etc 

reg r_oe_stop ;
reg r_oe_trdy ;
reg r_oe_devsel ;
reg r_oe_ack64 ;
reg r_oe_AD_hi ;
reg r_oe_AD_lo ;

reg r_stop_N ;
reg r_trdy_N ;
reg r_devsel_N ;
reg r_ack64_N ;

reg oe_par ;
reg oe_par64 ;

reg xfer_64_flag ;   // will be used for asserting O_data_valid_hi ;
reg sig_1 ;
reg [31:0] r_data_from_cfg; 

// take in the inputs form the PCI interface 

assign O_in_data_bus_hi = IO_AD [63:32] ;
assign O_in_data_bus_lo = IO_AD [31:0 ] ;
assign O_in_par = IO_PAR ;
assign O_in_par64 = IO_PAR64 ;

assign O_tgt_do_rd_xfer = I_read_mode & I_trf_not_full ;
assign O_incr_addr_to_app = (O_reg_tgt_wr_nxt | (O_tgt_do_rd_xfer && I_data_valid) ) ;


// Generate a fifo reset reset pulse aligned to app_clk ( for trf)
// looks like the asynchronous set is unavoidable 
always@(negedge I_RST_N or posedge I_end_xfer or posedge I_app_clk) 

begin
if (!I_RST_N)
   O_fifo_reset <= 1'b0 ;
else if (I_end_xfer)
    
   O_fifo_reset <= 1'b1 ; // asynchronouse set  -- check about read mode later 
else 
   O_fifo_reset <= (I_read_mode & I_end_xfer );
end 

// register all signals 

always@(posedge I_app_clk or negedge I_RST_N)
begin 
if (!I_RST_N) 
   O_reg_tgt_wr_nxt <= 1'b0 ;
else
   O_reg_tgt_wr_nxt <= I_tgt_wr_nxt ;

end

always@(posedge I_CLK or negedge I_RST_N)
begin 
if (!I_RST_N) 
   xfer_64_flag <= 1'b0 ;
else
   if (!I_FRAME_N && !I_REQ64_N) 
      xfer_64_flag <= 1'b1 ; // set condition
   else if (!I_FRAME_N && I_REQ64_N)
      xfer_64_flag <= 1'b0 ; // at the next non 64 command 
end 



// generate the data valid signals 

always@(posedge I_app_clk or negedge I_RST_N or negedge I_tgt_wr_data_avail)
begin
//if ((!I_RST_N ) || (!I_tgt_wr_data_avail) )   Gives synthesis errors
if (!I_RST_N) 
   begin
      O_data_valid_lo <= 1'b0 ; // worry about hi later
      O_data_valid_hi <= 1'b0 ;
   end 

else if (!I_tgt_wr_data_avail)

   begin
      O_data_valid_lo <= 1'b0 ;
      O_data_valid_hi <= 1'b0 ;
   end 


else
   begin
      if (I_tgt_wr_nxt)
         begin 
            O_data_valid_hi <= xfer_64_flag;
            O_data_valid_lo <= 1'b1 ;
         end 
      else
         begin
            O_data_valid_lo <= 1'b0 ;
            O_data_valid_hi <= 1'b0 ;
         end 
   end 
end

//



// - 2 cycles delay for writes to be provided for balancing 
always@(posedge I_CLK or negedge I_RST_N)
begin

if (!I_RST_N)
   begin
      sig_1 <= 1'b0 ;
      O_r_incr_twf_wrptr <= 1'b0 ;
   end 
else
   begin
      sig_1 <= I_incr_twf_wrptr ;
      O_r_incr_twf_wrptr <= sig_1 ;
   end 
end 

// Data from cfg also should be registered with PCI clock edge 
// for right time of arrival with fifo data path 

always@(negedge I_RST_N or posedge I_CLK)
begin

if (!I_RST_N)
   r_data_from_cfg <= 32'b0 ;
else
   r_data_from_cfg <= I_data_out_from_cfg ;
end 


// The following assignments handle the tristating at the PCI interface

assign O_STOP_N  = (r_oe_stop) ? r_stop_N : 1'bz ;
assign O_TRDY_N  = (r_oe_trdy) ? r_trdy_N : 1'bz ;


assign O_DEVSEL_N  = (r_oe_devsel) ? r_devsel_N : 1'bz ;
assign O_ACK64_N  = (r_oe_ack64) ? r_ack64_N : 1'bz ;  

//assign out_data_bus_lo = (I_valid_cfg_rd) ? I_data_out_from_cfg : I_rd_fifo_data_lo ;
assign out_data_bus_lo = (I_valid_cfg_rd) ? r_data_from_cfg : I_rd_fifo_data_lo ;

// 64 bit data path in use 
assign IO_AD[63:32] = (r_oe_AD_hi) ?  I_rd_fifo_data_hi : 32'bz ; // Pass fifo data hi when

// probably config data has to be delayed by a cycle 
assign IO_AD[31:0] = (r_oe_AD_lo |(~r_devsel_N & I_valid_cfg_rd) ) ?  out_data_bus_lo : 32'bz;


assign IO_PAR = (oe_par) ? I_calcd_par : 1'bz ;
assign IO_PAR64 = (oe_par64) ? I_calcd_par64 : 1'bz ;

// register devsel,trdy with proper end condition  - Test code 

always@(negedge I_RST_N or posedge I_CLK)
begin

if (!I_RST_N)
   begin
      r_devsel_N <= 1'b1 ;
      r_oe_devsel<= 1'b0 ;
      r_trdy_N <= 1'b1 ;
      r_oe_trdy<= 1'b0 ;
      r_oe_ack64  <= 1'b0;
      r_stop_N    <= 1'b1 ; // deasserted  
      r_oe_stop   <= 1'b0;  // disabled 
   end 
else
   begin
      if (I_FRAME_N && !r_trdy_N &&  !I_IRDY_N && !I_single_cyc_xfer) // last cycle over
         begin 
            r_devsel_N <= 1'b1 ;
            r_ack64_N   <= 1'b1 ;
            r_trdy_N <= 1'b1 ;
            r_oe_devsel <= 1'b1 ;
            r_oe_trdy <= 1'b1 ;
            r_oe_ack64  <= 1'b1  ;
            r_stop_N    <= 1'b1 ;
            r_oe_stop   <= 1'b1; 
         end 
      else // in single cyc xfers, do not have to take the signals early
         begin
            r_devsel_N <= I_devsel_N ;
            r_trdy_N <= I_trdy_N ;
            r_ack64_N   <= I_ack64_N  ;
            r_oe_devsel <= I_oe_devsel ;
            r_oe_trdy <= I_oe_trdy ;
            r_oe_ack64  <= I_oe_ack64  ;
            r_stop_N    <= I_stop_N   ;
            r_oe_stop   <= I_oe_stop  ;
         end
   end 
end 

// Generated data enables 

always@(negedge I_RST_N or posedge I_CLK)
begin

if (!I_RST_N)
   begin
      r_oe_AD_hi  <= 1'b0 ;
      r_oe_AD_lo  <= 1'b0 ;
   end
else
   begin
      if (I_FRAME_N && !r_trdy_N &&  !I_IRDY_N && !I_single_cyc_xfer) // last cycle over
         begin 
            r_oe_AD_hi  <= 1'b0 ;
            r_oe_AD_lo  <= 1'b0 ;
         end
      else
         begin
            r_oe_AD_hi  <= I_oe_AD_hi ;
            r_oe_AD_lo  <= I_oe_AD_lo ;
         end 
   end 
end 

// register stop and its enable ; stop should be deasserted as soon as Frame goes high

/*always@(negedge I_RST_N or posedge I_CLK)
begin

if (!I_RST_N)
   begin
      r_stop_N    <= 1'b1 ; // deasserted  
      r_oe_stop   <= 1'b0;  // disabled 
   end
else
   if (I_FRAME_N && !I_single_cyc_xfer) // frame has been de-asserted 
      begin
         r_stop_N    <= 1'b1   ;
         r_oe_stop   <= 1'b0  ; // deasserted values
      end
   else
      begin
         r_stop_N    <= I_stop_N   ;
         r_oe_stop   <= I_oe_stop  ;
      end

      begin
         r_stop_N    <= I_stop_N   ;
         r_oe_stop   <= I_oe_stop  ;
      end



end 
*/


// Generate enables for parity out 

always@(posedge I_CLK or negedge I_RST_N)
begin
if (!I_RST_N)
   begin
      oe_par <= 1'b0 ;
      oe_par64 <= 1'b0 ;
   end 
else
   begin
      oe_par <= r_oe_AD_lo ;
      oe_par64 <= r_oe_AD_hi ;
   end
end 

endmodule 
